@@ -1,13 +1,16 @@
 """
 Series class
 """
+
 from abc import ABC, abstractmethod
 from bisect import bisect_left, bisect_right
 from collections import OrderedDict
 from itertools import compress
+from typing import Any, Callable, Literal, Self
 
 from tabulate import tabulate
 
+from raccoon import DataFrame
 from raccoon.sort_utils import sorted_exists, sorted_index, sorted_list_indexes
 
 
@@ -16,34 +19,35 @@ class SeriesBase(ABC):
     Base Series abstract base class that concrete implementations inherit from. Note that the .data and .index property
     methods in Series are views to the underlying data and not copies.
     """
+
     # Define slots to make object faster
-    __slots__ = ['_data', '_data_name', '_index', '_index_name', '_sort', '_dropin']
+    __slots__ = ["_data", "_data_name", "_index", "_index_name", "_sort", "_dropin"]
 
     def __init__(self):
         """
         No specific parameters, those are defined in the child classed
         """
-        self._index = None
-        self._index_name = None
-        self._data = None
-        self._data_name = None
-        self._sort = None
-        self._dropin = None
+        self._index: list | None = None
+        self._index_name: str | tuple | None = None
+        self._data: list | None = None
+        self._data_name: str | tuple | None = None
+        self._sort: bool | None = None
+        self._dropin: Callable = None
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._index)
 
-    def __repr__(self):
-        return 'object id: %s\ndata:\n%s\nindex:\n%s\n' % (id(self), self._data, self._index)
+    def __repr__(self) -> str:
+        return "object id: %s\ndata:\n%s\nindex:\n%s\n" % (id(self), self._data, self._index)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self._make_table()
 
-    def _make_table(self, index=True, **kwargs):
-        kwargs['headers'] = 'keys' if 'headers' not in kwargs.keys() else kwargs['headers']
+    def _make_table(self, index: bool = True, **kwargs) -> str:
+        kwargs["headers"] = "keys" if "headers" not in kwargs.keys() else kwargs["headers"]
         return tabulate(self.to_dict(ordered=True, index=index), **kwargs)
 
-    def print(self, index=True, **kwargs):
+    def print(self, index: bool = True, **kwargs) -> None:
         """
         Print the contents of the Series. This method uses the tabulate function from the tabulate package. Use the
         kwargs to pass along any arguments to the tabulate function.
@@ -70,19 +74,19 @@ class SeriesBase(ABC):
         return
 
     @property
-    def data_name(self):
+    def data_name(self) -> str | tuple | None:
         return self._data_name
 
     @data_name.setter
-    def data_name(self, name):
+    def data_name(self, name: str | tuple | None) -> None:
         self._data_name = name
 
     @property
-    def index_name(self):
+    def index_name(self) -> str | tuple | None:
         return self._index_name
 
     @index_name.setter
-    def index_name(self, name):
+    def index_name(self, name: str | tuple | None) -> None:
         self._index_name = name
 
     @property
@@ -90,10 +94,10 @@ class SeriesBase(ABC):
     def sort(self):
         return
 
-    def _check_list(self, x):
+    def _check_list(self, x: Any) -> bool:
         return type(x) == (self._dropin if self._dropin else list)
 
-    def get(self, indexes, as_list=False):
+    def get(self, indexes: Any | list | list[bool], as_list: bool = False) -> Self | list | Any:
         """
         Given indexes will return a sub-set of the Series. This method will direct to the specific methods
         based on what types are passed in for the indexes. The type of the return is determined by the
@@ -108,7 +112,7 @@ class SeriesBase(ABC):
         else:
             return self.get_cell(indexes)
 
-    def get_cell(self, index):
+    def get_cell(self, index: Any) -> Any:
         """
         For a single index and return the value
 
@@ -118,7 +122,7 @@ class SeriesBase(ABC):
         i = sorted_index(self._index, index) if self._sort else self._index.index(index)
         return self._data[i]
 
-    def get_rows(self, indexes, as_list=False):
+    def get_rows(self, indexes: Any | list | list[bool], as_list: bool = False) -> Self | list:
         """
         For a list of indexes return the values of the indexes in that column.
 
@@ -128,7 +132,7 @@ class SeriesBase(ABC):
         """
         if all([isinstance(i, bool) for i in indexes]):  # boolean list
             if len(indexes) != len(self._index):
-                raise ValueError('boolean index list must be same size of existing index')
+                raise ValueError("boolean index list must be same size of existing index")
             if all(indexes):  # the entire column
                 data = self._data
                 index = self._index
@@ -136,17 +140,30 @@ class SeriesBase(ABC):
                 data = list(compress(self._data, indexes))
                 index = list(compress(self._index, indexes))
         else:  # index values list
-            locations = [sorted_index(self._index, x) for x in indexes] if self._sort \
+            locations = (
+                [sorted_index(self._index, x) for x in indexes]
+                if self._sort
                 else [self._index.index(x) for x in indexes]
+            )
             data = [self._data[i] for i in locations]
             index = [self._index[i] for i in locations]
-        return data if as_list else Series(data=data, index=index, data_name=self._data_name,
-                                           index_name=self._index_name, sort=self._sort, dropin=self._dropin)
+        return (
+            data
+            if as_list
+            else Series(
+                data=data,
+                index=index,
+                data_name=self._data_name,
+                index_name=self._index_name,
+                sort=self._sort,
+                dropin=self._dropin,
+            )
+        )
 
-    def get_location(self, location):
+    def get_location(self, location: int) -> dict:
         """
         For an index location return a dict of the index and value. This is optimized for speed because
-        it does not need to lookup the index location with a search. Also can accept relative indexing from the end of
+        it does not need to look up the index location with a search. Also, can accept relative indexing from the end of
         the SEries in standard python notation [-3, -2, -1]
 
         :param location: index location in standard python form of positive or negative number
@@ -154,7 +171,7 @@ class SeriesBase(ABC):
         """
         return {self.index_name: self._index[location], self.data_name: self._data[location]}
 
-    def get_locations(self, locations, as_list=False):
+    def get_locations(self, locations: list[int], as_list: bool = False) -> Self | list:
         """
         For list of locations return a Series or list of the values.
 
@@ -166,9 +183,11 @@ class SeriesBase(ABC):
         indexes = [self._index[x] for x in locations]
         return self.get(indexes, as_list)
 
-    def get_slice(self, start_index=None, stop_index=None, as_list=False):
+    def get_slice(
+        self, start_index: Any = None, stop_index: Any = None, as_list: bool = False
+    ) -> Self | tuple[list, list]:
         """
-        For sorted Series will return either a Series or list of all of the rows where the index is greater than
+        For sorted Series will return either a Series or list of all the rows where the index is greater than
         or equal to the start_index if provided and less than or equal to the stop_index if provided. If either the
         start or stop index is None then will include from the first or last element, similar to standard python
         slide of [:5] or [:5]. Both end points are considered inclusive.
@@ -179,7 +198,7 @@ class SeriesBase(ABC):
         :return: Series or tuple of (index list, values list)
         """
         if not self._sort:
-            raise RuntimeError('Can only use get_slice on sorted Series')
+            raise RuntimeError("Can only use get_slice on sorted Series")
 
         start_location = bisect_left(self._index, start_index) if start_index is not None else None
         stop_location = bisect_right(self._index, stop_index) if stop_index is not None else None
@@ -190,20 +209,26 @@ class SeriesBase(ABC):
         if as_list:
             return index, data
         else:
-            return Series(data=data, index=index, data_name=self._data_name, index_name=self._index_name,
-                          sort=self._sort, dropin=self._dropin)
+            return Series(
+                data=data,
+                index=index,
+                data_name=self._data_name,
+                index_name=self._index_name,
+                sort=self._sort,
+                dropin=self._dropin,
+            )
 
-    def _slice_index(self, slicer):
+    def _slice_index(self, slicer: slice) -> list:
         try:
             start_index = sorted_index(self._index, slicer.start) if self._sort else self._index.index(slicer.start)
         except ValueError:
-            raise IndexError('start of slice not in the index')
+            raise IndexError("start of slice not in the index")
         try:
             end_index = sorted_index(self._index, slicer.stop) if self._sort else self._index.index(slicer.stop)
         except ValueError:
-            raise IndexError('end of slice not in the index')
+            raise IndexError("end of slice not in the index")
         if end_index < start_index:
-            raise IndexError('end of slice is before start of slice')
+            raise IndexError("end of slice is before start of slice")
 
         pre_list = [False] * start_index
         mid_list = [True] * (end_index - start_index + 1)
@@ -213,7 +238,7 @@ class SeriesBase(ABC):
         pre_list.extend(post_list)
         return pre_list
 
-    def _validate_index(self, indexes):
+    def _validate_index(self, indexes: list) -> None:
         """
         Raises an error if the indexes are not valid
 
@@ -221,14 +246,14 @@ class SeriesBase(ABC):
         :return: nothing
         """
         if not (self._check_list(indexes) or type(indexes) == list or indexes is None):
-            raise TypeError('indexes must be list, %s or None' % self._dropin)
+            raise TypeError("indexes must be list, %s or None" % self._dropin)
         if len(indexes) != len(set(indexes)):  # noqa
-            raise ValueError('index contains duplicates')
+            raise ValueError("index contains duplicates")
         if self._data:
             if len(indexes) != len(self._data):  # noqa
-                raise ValueError('index length does not match data length')
+                raise ValueError("index length does not match data length")
 
-    def validate_integrity(self):
+    def validate_integrity(self) -> None:
         """
         Validate the integrity of the Series. This checks that the indexes, column names and internal data are not
         corrupted. Will raise an error if there is a problem.
@@ -237,7 +262,7 @@ class SeriesBase(ABC):
         """
         self._validate_index(self._index)
 
-    def to_dict(self, index=True, ordered=False):
+    def to_dict(self, index: bool = True, ordered: bool = False) -> dict:
         """
         Returns a dict where the keys are the data and index names and the values are list of the data and index.
 
@@ -255,7 +280,7 @@ class SeriesBase(ABC):
         result.update(data_dict)
         return result
 
-    def head(self, rows):
+    def head(self, rows: int) -> Self:
         """
         Return a Series of the first N rows
 
@@ -266,7 +291,7 @@ class SeriesBase(ABC):
         rows_bool.extend([False] * max(0, len(self._index) - rows))
         return self.get(indexes=rows_bool)
 
-    def tail(self, rows):
+    def tail(self, rows: int) -> Self:
         """
         Return a Series of the last N rows
 
@@ -277,7 +302,7 @@ class SeriesBase(ABC):
         rows_bool.extend([True] * min(rows, len(self._index)))
         return self.get(indexes=rows_bool)
 
-    def select_index(self, compare, result='boolean'):
+    def select_index(self, compare: Any | tuple, result: Literal["boolean", "value"] = "boolean") -> list[bool | Any]:
         """
         Finds the elements in the index that match the compare parameter and returns either a list of the values that
         match, of a boolean list the length of the index with True to each index that matches. If the indexes are
@@ -290,31 +315,33 @@ class SeriesBase(ABC):
         """
         if isinstance(compare, tuple):
             # this crazy list comprehension will match all the tuples in the list with None being an * wildcard
-            booleans = [all([(compare[i] == w if compare[i] is not None else True) for i, w in enumerate(v)])
-                        for x, v in enumerate(self._index)]
+            booleans = [
+                all([(compare[i] == w if compare[i] is not None else True) for i, w in enumerate(v)])
+                for x, v in enumerate(self._index)
+            ]
         else:
             booleans = [False] * len(self._index)
             if self._sort:
                 booleans[sorted_index(self._index, compare)] = True
             else:
                 booleans[self._index.index(compare)] = True
-        if result == 'boolean':
+        if result == "boolean":
             return booleans
-        elif result == 'value':
+        elif result == "value":
             return list(compress(self._index, booleans))
         else:
-            raise ValueError('only valid values for result parameter are: boolean or value.')
+            raise ValueError("only valid values for result parameter are: boolean or value.")
 
-    def isin(self, compare_list):
+    def isin(self, compare_list: list) -> list[bool]:
         """
-        Returns a boolean list where each elements is whether that element in the column is in the compare_list.
+        Returns a boolean list where each element is whether that element in the column is in the compare_list.
 
         :param compare_list: list of items to compare to
         :return: list of booleans
         """
         return [x in compare_list for x in self._data]
 
-    def equality(self, indexes=None, value=None):
+    def equality(self, indexes: list | list[bool] = None, value: Any = None) -> list[bool]:
         """
         Math helper method. Given a column and optional indexes will return a list of booleans on the equality of the
         value for that index in the DataFrame to the value parameter.
@@ -340,13 +367,22 @@ class Series(SeriesBase):
     index remains sort.
     """
 
-    def __init__(self, data=None, index=None, data_name='value', index_name='index', sort=None, dropin=None):
+    def __init__(
+        self,
+        data: dict | list | None = None,
+        index: list | None = None,
+        data_name: str | tuple | None = "value",
+        index_name: str | tuple | None = "index",
+        sort: bool = None,
+        dropin: Callable = None,
+    ):
         """
         :param data: (optional) list of values.
         :param index: (optional) list of index values. If None then the index will be integers starting with zero
         :param data_name: (optional) name of the data column, or will default to 'value'
         :param index_name: (optional) name for the index. Default is "index"
-        :param sort: if True then Series will keep the index sort. If True all index values must be of same type
+        :param sort: if True then Series will keep the index sort. If True all index values must be of same type. If
+            None then will default to True if no index is provided.
         :param dropin: if supplied the drop-in replacement for list that will be used
         """
         super(SeriesBase, self).__init__()
@@ -375,7 +411,7 @@ class Series(SeriesBase):
             else:
                 self.index = list(range(len(self._data)))
         else:
-            raise TypeError('Not valid data type.')
+            raise TypeError("Not valid data type.")
 
         # setup sort
         self._sort = None
@@ -387,7 +423,7 @@ class Series(SeriesBase):
             else:
                 self.sort = True
 
-    def _pad_data(self, index_len):
+    def _pad_data(self, index_len: int) -> None:
         """
         Pad the data in Series with [None] to ensure that data is the same length as index
 
@@ -397,33 +433,33 @@ class Series(SeriesBase):
         self._data.extend([None] * (index_len - len(self._data)))
 
     @property
-    def data(self):
+    def data(self) -> list:
         return self._data
 
     @property
-    def index(self):
+    def index(self) -> list:
         return self._index
 
     @index.setter
-    def index(self, index_list):
+    def index(self, index_list: list) -> None:
         self._validate_index(index_list)
         self._index = self._dropin(index_list) if self._dropin else list(index_list)
 
     @property
-    def dropin(self):
+    def dropin(self) -> Callable:
         return self._dropin
 
     @property
-    def sort(self):
+    def sort(self) -> bool:
         return self._sort
 
     @sort.setter
-    def sort(self, boolean):
+    def sort(self, boolean: bool) -> None:
         self._sort = boolean
         if self._sort:
             self.sort_index()
 
-    def sort_index(self):
+    def sort_index(self) -> None:
         """
         Sort the Series by the index. The sort modifies the Series inplace
 
@@ -435,14 +471,14 @@ class Series(SeriesBase):
         # sort data
         self._data = self._dropin([self._data[x] for x in sort]) if self._dropin else [self._data[x] for x in sort]
 
-    def set(self, indexes, values=None):
+    def set(self, indexes: Any | list, values: Any | list = None) -> None:
         """
         Given indexes will set a sub-set of the Series to the values provided. This method will direct to the below
-        methods based on what types are passed in for the indexes. If the indexes contains values not in the Series
+        methods based on what types are passed in for the indexes. If the indexes contain values not in the Series
         then new rows or columns will be added.
 
         :param indexes: indexes value, list of indexes values, or a list of booleans.
-        :param values: value or list of values to set. If a list then must be the same length as the indexes parameter.
+        :param values: value or list of values to set. If a list then must be the same length as the index's parameter.
         :return: nothing
         """
         if self._check_list(indexes):
@@ -450,7 +486,7 @@ class Series(SeriesBase):
         else:
             self.set_cell(indexes, values)
 
-    def _add_row(self, index):
+    def _add_row(self, index: Any) -> None:
         """
         Add a new row to the Series
 
@@ -460,7 +496,7 @@ class Series(SeriesBase):
         self._index.append(index)
         self._data.append(None)
 
-    def _insert_row(self, i, index):
+    def _insert_row(self, i: int, index: Any) -> None:
         """
         Insert a new row in the Series.
 
@@ -474,7 +510,7 @@ class Series(SeriesBase):
             self._index.insert(i, index)
             self._data.insert(i, None)
 
-    def _add_missing_rows(self, indexes):
+    def _add_missing_rows(self, indexes: list) -> None:
         """
         Given a list of indexes, find all the indexes that are not currently in the Series and make a new row for
         that index by appending to the Series. This does not maintain sorted order for the index.
@@ -486,7 +522,7 @@ class Series(SeriesBase):
         for x in new_indexes:
             self._add_row(x)
 
-    def _insert_missing_rows(self, indexes):
+    def _insert_missing_rows(self, indexes: list) -> None:
         """
         Given a list of indexes, find all the indexes that are not currently in the Series and make a new row for
         that index, inserting into the index. This requires the Series to be sorted=True
@@ -498,7 +534,7 @@ class Series(SeriesBase):
         for x in new_indexes:
             self._insert_row(bisect_left(self._index, x), x)
 
-    def set_cell(self, index, value):
+    def set_cell(self, index: Any, value: Any) -> None:
         """
         Sets the value of a single cell. If the index is not in the current index then a new index will be created.
 
@@ -518,7 +554,7 @@ class Series(SeriesBase):
                 self._add_row(index)
         self._data[i] = value
 
-    def set_rows(self, index, values=None):
+    def set_rows(self, index: list | list[bool], values: Any | list = None) -> None:
         """
         Set rows to a single value or list of values. If any of the index values are not in the current indexes
         then a new row will be created.
@@ -533,9 +569,9 @@ class Series(SeriesBase):
             if not self._check_list(values):  # single value provided, not a list, so turn values into list
                 values = [values for x in index if x]
             if len(index) != len(self._index):
-                raise ValueError('boolean index list must be same size of existing index')
+                raise ValueError("boolean index list must be same size of existing index")
             if len(values) != index.count(True):
-                raise ValueError('length of values list must equal number of True entries in index list')
+                raise ValueError("length of values list must equal number of True entries in index list")
             indexes = [i for i, x in enumerate(index) if x]
             for x, i in enumerate(indexes):
                 self._data[i] = values[x]
@@ -543,7 +579,7 @@ class Series(SeriesBase):
             if not self._check_list(values):  # single value provided, not a list, so turn values into list
                 values = [values for _ in index]
             if len(values) != len(index):
-                raise ValueError('length of values and index must be the same.')
+                raise ValueError("length of values and index must be the same.")
             # insert or append indexes as needed
             if self._sort:
                 exists_tuples = list(zip(*[sorted_exists(self._index, x) for x in index]))
@@ -561,7 +597,7 @@ class Series(SeriesBase):
             for x, i in enumerate(indexes):
                 self._data[i] = values[x]
 
-    def set_location(self, location, value):
+    def set_location(self, location: int, value: Any) -> None:
         """
         For a location set the value
 
@@ -571,7 +607,7 @@ class Series(SeriesBase):
         """
         self._data[location] = value
 
-    def set_locations(self, locations, values):
+    def set_locations(self, locations: list[int], values: list | Any) -> None:
         """
         For a list of locations set the values.
 
@@ -583,7 +619,7 @@ class Series(SeriesBase):
         indexes = [self._index[x] for x in locations]
         self.set(indexes, values)
 
-    def __setitem__(self, index, value):
+    def __setitem__(self, index: Any | list | slice, value: Any | list) -> None:
         """
         Convenience wrapper around the set() method for using srs[] = X
         Usage...
@@ -599,7 +635,7 @@ class Series(SeriesBase):
         indexes = self._slice_index(index) if isinstance(index, slice) else index
         return self.set(indexes=indexes, values=value)
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: Any | list | slice) -> Any:
         """
         Convenience wrapper around the get() method for using srs[]
         Usage...
@@ -619,7 +655,7 @@ class Series(SeriesBase):
         else:  # just a single cell or list of cells
             return self.get(index)
 
-    def append_row(self, index, value):
+    def append_row(self, index: Any, value: Any) -> None:
         """
         Appends a row of value to the end of the data. Be very careful with this function as for sorted Series it will
         not enforce sort order. Use this only for speed when needed, be careful.
@@ -629,12 +665,12 @@ class Series(SeriesBase):
         :return: nothing
         """
         if index in self._index:
-            raise IndexError('index already in Series')
+            raise IndexError("index already in Series")
 
         self._index.append(index)
         self._data.append(value)
 
-    def append_rows(self, indexes, values):
+    def append_rows(self, indexes: list, values: list) -> None:
         """
         Appends values to the end of the data. Be very careful with this function as for sort DataFrames it will not
         enforce sort order. Use this only for speed when needed, be careful.
@@ -646,18 +682,18 @@ class Series(SeriesBase):
 
         # check that the values data is less than or equal to the length of the indexes
         if len(values) != len(indexes):
-            raise ValueError('length of values is not equal to length of indexes')
+            raise ValueError("length of values is not equal to length of indexes")
 
         # check the indexes are not duplicates
         combined_index = self._index + indexes
         if len(set(combined_index)) != len(combined_index):
-            raise IndexError('duplicate indexes in Series')
+            raise IndexError("duplicate indexes in Series")
 
         # append index value
         self._index.extend(indexes)
         self._data.extend(values)
 
-    def delete(self, indexes):
+    def delete(self, indexes: Any | list | list[bool]) -> None:
         """
         Delete rows from the DataFrame
 
@@ -667,11 +703,14 @@ class Series(SeriesBase):
         indexes = [indexes] if not self._check_list(indexes) else indexes
         if all([isinstance(i, bool) for i in indexes]):  # boolean list
             if len(indexes) != len(self._index):
-                raise ValueError('boolean indexes list must be same size of existing indexes')
+                raise ValueError("boolean indexes list must be same size of existing indexes")
             indexes = [i for i, x in enumerate(indexes) if x]
         else:
-            indexes = [sorted_index(self._index, x) for x in indexes] if self._sort \
+            indexes = (
+                [sorted_index(self._index, x) for x in indexes]
+                if self._sort
                 else [self._index.index(x) for x in indexes]
+            )
         indexes = sorted(indexes, reverse=True)  # need to sort and reverse list so deleting works
         for i in indexes:
             del self._data[i]
@@ -679,14 +718,14 @@ class Series(SeriesBase):
         for i in indexes:
             del self._index[i]
 
-    def reset_index(self):
+    def reset_index(self) -> None:
         """
         Resets the index of the Series to simple integer list and the index name to 'index'.
 
         :return: nothing
         """
         self.index = list(range(self.__len__()))
-        self.index_name = 'index'
+        self.index_name = "index"
 
 
 class ViewSeries(SeriesBase):
@@ -697,7 +736,15 @@ class ViewSeries(SeriesBase):
     checking of the data, so it is assumed the data type is list-style.
     """
 
-    def __init__(self, data=None, index=None, data_name='value', index_name='index', sort=False, offset=0):
+    def __init__(
+        self,
+        data: list | tuple | None = None,
+        index: list | None = None,
+        data_name: str | tuple | None = "value",
+        index_name: str | tuple | None = "index",
+        sort: bool = False,
+        offset: int = 0,
+    ):
         """
         :param data: (optional) list of values.
         :param index: (optional) list of index values. If None then the index will be integers starting with zero
@@ -713,9 +760,9 @@ class ViewSeries(SeriesBase):
 
         # check inputs
         if index is None:
-            raise ValueError('Index cannot be None.')
+            raise ValueError("Index cannot be None.")
         if data is None:
-            raise ValueError('Data cannot be None.')
+            raise ValueError("Data cannot be None.")
 
         # standard variable setup
         self._data = data  # direct view, no copy
@@ -726,27 +773,27 @@ class ViewSeries(SeriesBase):
         self._offset = offset
 
     @property
-    def data(self):
+    def data(self) -> list:
         return self._data
 
     @property
-    def index(self):
+    def index(self) -> list:
         return self._index
 
     @index.setter
-    def index(self, index_list):
+    def index(self, index_list: list) -> None:
         self._validate_index(index_list)
         self._index = index_list
 
     @property
-    def sort(self):
+    def sort(self) -> bool:
         return self._sort
 
     @property
-    def offset(self):
+    def offset(self) -> int:
         return self._offset
 
-    def value(self, indexes, int_as_index=False):
+    def value(self, indexes: int | Any | list | list[bool], int_as_index: bool = False) -> Any | list:
         """
         Wrapper function for get. It will return a list, no index. If the indexes are integers it will be assumed
         that they are locations unless int_as_index = True. If the indexes are locations then they will be rotated to
@@ -771,9 +818,9 @@ class ViewSeries(SeriesBase):
                 stop = indexes.stop - self._offset + 1  # to capture the last value
                 # check locations are valid and will not return empty
                 if start > stop:
-                    raise IndexError('end of slice is before start of slice')
+                    raise IndexError("end of slice is before start of slice")
                 if (start > 0 > stop) or (start < 0 < stop):
-                    raise IndexError('slide indexes invalid with given offset:%f' % self._offset)
+                    raise IndexError("slide indexes invalid with given offset:%f" % self._offset)
                 # where end is the last element
                 if (start < 0) and stop == 0:
                     return self._data[start:]
@@ -798,7 +845,7 @@ class ViewSeries(SeriesBase):
         else:
             return self.get(indexes)
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: Any | list | slice) -> Self:
         """
         Convenience wrapper around the value() method for using srs[]. This will treat all integers as locations
 
@@ -815,7 +862,7 @@ class ViewSeries(SeriesBase):
 
     # Series creation functions
     @classmethod
-    def from_dataframe(cls, dataframe, column, offset=0):
+    def from_dataframe(cls, dataframe: DataFrame, column: str | tuple | None, offset: int = 0) -> Self:
         """
         Creates and return a Series from a DataFrame and specific column
 
@@ -824,11 +871,17 @@ class ViewSeries(SeriesBase):
         :param offset: offset value must be provided as there is no equivalent for a DataFrame
         :return: Series
         """
-        return cls(data=dataframe.get_entire_column(column, as_list=True), index=dataframe.index,
-                   data_name=column, index_name=dataframe.index_name, sort=dataframe.sort, offset=offset)
+        return cls(
+            data=dataframe.get_entire_column(column, as_list=True),
+            index=dataframe.index,
+            data_name=column,
+            index_name=dataframe.index_name,
+            sort=dataframe.sort,
+            offset=offset,
+        )
 
     @classmethod
-    def from_series(cls, series, offset=0):
+    def from_series(cls, series: Series, offset: int = 0) -> Self:
         """
         Creates and return a Series from a Series
 
@@ -836,5 +889,11 @@ class ViewSeries(SeriesBase):
         :param offset: offset value must be provided as there is no equivalent for a DataFrame
         :return: Series
         """
-        return cls(data=series.data, index=series.index, data_name=series.data_name, index_name=series.index_name,
-                   sort=series.sort, offset=offset)
+        return cls(
+            data=series.data,
+            index=series.index,
+            data_name=series.data_name,
+            index_name=series.index_name,
+            sort=series.sort,
+            offset=offset,
+        )
